@@ -1,9 +1,11 @@
 /*global define*/
-define(function(require) {
+define(['require','tmpl!./setup_account_prefs.html','./account_prefs_mixins','mix','evt','l10n!','mail_common'],function(require) {
 
 var templateNode = require('tmpl!./setup_account_prefs.html'),
     prefsMixin = require('./account_prefs_mixins'),
     mix = require('mix'),
+    evt = require('evt'),
+    mozL10n = require('l10n!'),
     common = require('mail_common'),
     Cards = common.Cards;
 
@@ -13,11 +15,20 @@ var templateNode = require('tmpl!./setup_account_prefs.html'),
 function SetupAccountPrefsCard(domNode, mode, args) {
   this.domNode = domNode;
   this.account = args.account;
+  this.identity = this.account.identities[0];
+
+  // Establish defaults specifically for our email app.
+  this.identity.modifyIdentity({ signatureEnabled: true,
+    signature: mozL10n.get('settings-default-signature-2') });
 
   this.nextButton = this.nodeFromClass('sup-info-next-btn');
   this.nextButton.addEventListener('click', this.onNext.bind(this), false);
 
-  this._bindPrefs('tng-account-check-interval', 'tng-notify-mail');
+  this._bindPrefs('tng-account-check-interval',
+                  'tng-notify-mail',
+                  'tng-sound-onsend',
+                  'tng-signature-input',
+                  'signature-button');
 }
 
 SetupAccountPrefsCard.prototype = {
@@ -27,6 +38,10 @@ SetupAccountPrefsCard.prototype = {
       {});
   },
 
+  onCardVisible: function() {
+    this.updateSignatureButton();
+  },
+
   die: function() {
   }
 };
