@@ -146,10 +146,10 @@ var DragDropManager = (function() {
             pageHelper.addPage([draggableIcon]);
           }
 
+          setDisabledCheckingLimits(true);
           if (pageHelper.getNext()) {
             transitioning = true;
             GridManager.goToNextPage(onNavigationEnd);
-            setDisabledCheckingLimits(true);
           }
 
           done();
@@ -401,7 +401,7 @@ var DragDropManager = (function() {
   }
 
   function move() {
-    draggableElemStyle.MozTransform =
+    draggableElemStyle.transform =
       'translate(' + (cx - sx) + 'px,' + (cy - sy) + 'px)';
   }
 
@@ -518,11 +518,8 @@ var DragDropManager = (function() {
     clearOverCollectionTimeout();
     window.removeEventListener(touchmove, onMove);
     window.removeEventListener(touchend, onEnd);
-
-    GridManager.onDragStop();
-
     stop(function dg_stop() {
-      DockManager.onDragStop();
+      DockManager.onDragStop(GridManager.onDragStop);
       window.dispatchEvent(new CustomEvent('dragend'));
     });
   }
