@@ -1,3 +1,4 @@
+/* exported BlobView */
 'use strict';
 
 var BlobView = (function() {
@@ -32,17 +33,19 @@ var BlobView = (function() {
 
   // Async factory function
   BlobView.get = function(blob, offset, length, callback, littleEndian) {
-    if (offset < 0)
+    if (offset < 0) {
       fail('negative offset');
-    if (length < 0)
+    }
+    if (length < 0) {
       fail('negative length');
-    if (offset > blob.size)
+    }
+    if (offset > blob.size) {
       fail('offset larger than blob size');
-
+    }
     // Don't fail if the length is too big; just reduce the length
-    if (offset + length > blob.size)
+    if (offset + length > blob.size) {
       length = blob.size - offset;
-
+    }
     var slice = blob.slice(offset, offset + length);
     var reader = new FileReader();
     reader.readAsArrayBuffer(slice);
@@ -172,23 +175,27 @@ var BlobView = (function() {
       return this.byteLength - this.index;
     },
     seek: function(index) {
-      if (index < 0)
+      if (index < 0) {
         fail('negative index');
-      if (index > this.byteLength)
+      }
+      if (index > this.byteLength) {
         fail('index greater than buffer size');
+      }
       this.index = index;
     },
     advance: function(n) {
       var index = this.index + n;
-      if (index < 0)
+      if (index < 0) {
         fail('advance past beginning of buffer');
+      }
       // It's usual that when we finished reading one target view,
       // the index is advanced to the start(previous end + 1) of next view,
       // and the new index will be equal to byte length(the last index + 1),
       // we will not fail on it because it means the reading is finished,
       // or do we have to warn here?
-      if (index > this.byteLength)
+      if (index > this.byteLength) {
         fail('advance past end of buffer');
+      }
       this.index = index;
     },
 
@@ -273,42 +280,51 @@ var BlobView = (function() {
         }
         else if (b1 < 224) {
           // 2-byte sequence
-          if (pos + 1 >= end)
+          if (pos + 1 >= end) {
             fail();
+          }
           b2 = this.view.getUint8(pos + 1);
-          if (b2 < 128 || b2 > 191)
+          if (b2 < 128 || b2 > 191) {
             fail();
+          }
           charcode = ((b1 & 0x1f) << 6) + (b2 & 0x3f);
           s += String.fromCharCode(charcode);
           pos += 2;
         }
         else if (b1 < 240) {
           // 3-byte sequence
-          if (pos + 2 >= end)
+          if (pos + 2 >= end) {
             fail();
+          }
           b2 = this.view.getUint8(pos + 1);
-          if (b2 < 128 || b2 > 191)
+          if (b2 < 128 || b2 > 191) {
             fail();
+          }
           b3 = this.view.getUint8(pos + 2);
-          if (b3 < 128 || b3 > 191)
+          if (b3 < 128 || b3 > 191) {
             fail();
+          }
           charcode = ((b1 & 0x0f) << 12) + ((b2 & 0x3f) << 6) + (b3 & 0x3f);
           s += String.fromCharCode(charcode);
           pos += 3;
         }
         else if (b1 < 245) {
           // 4-byte sequence
-          if (pos + 3 >= end)
+          if (pos + 3 >= end) {
             fail();
+          }
           b2 = this.view.getUint8(pos + 1);
-          if (b2 < 128 || b2 > 191)
+          if (b2 < 128 || b2 > 191) {
             fail();
+          }
           b3 = this.view.getUint8(pos + 2);
-          if (b3 < 128 || b3 > 191)
+          if (b3 < 128 || b3 > 191) {
             fail();
+          }
           b4 = this.view.getUint8(pos + 3);
-          if (b4 < 128 || b4 > 191)
+          if (b4 < 128 || b4 > 191) {
             fail();
+          }
           charcode = ((b1 & 0x07) << 18) +
             ((b2 & 0x3f) << 12) +
             ((b3 & 0x3f) << 6) +
@@ -376,12 +392,14 @@ var BlobView = (function() {
     // more than size bytes.  And return as a UTF8 string
     readNullTerminatedUTF8Text: function(size) {
       for (var len = 0; len < size; len++) {
-        if (this.view.getUint8(this.index + len) === 0)
+        if (this.view.getUint8(this.index + len) === 0) {
           break;
+        }
       }
       var s = this.readUTF8Text(len);
-      if (len < size)    // skip the null terminator if we found one
+      if (len < size) {    // skip the null terminator if we found one
         this.advance(1);
+      }
       return s;
     },
 
@@ -392,10 +410,11 @@ var BlobView = (function() {
       if (le == null) {
         var BOM = this.readUnsignedShort();
         size -= 2;
-        if (BOM === 0xFEFF)
+        if (BOM === 0xFEFF) {
           le = false;
-        else
+        } else {
           le = true;
+        }
       }
 
       var s = '';
@@ -418,6 +437,7 @@ var BlobView = (function() {
   // real constructor, the contructor property of the prototype refers to it.
   return { get: BlobView.get };
 }());
+
 'use strict';
 
 //
@@ -501,6 +521,12 @@ function parseJPEGMetadata(file, metadataCallback, metadataError) {
         metadata.height = data.getUint16(5);
         metadata.width = data.getUint16(7);
 
+        if (type === 0xC2) {
+          // pjpeg files can't be efficiently downsampled while decoded
+          // so we need to distinguish them from regular jpegs
+          metadata.progressive = true;
+        }
+
         // We're done. All the EXIF data will come before this segment
         // So call the callback
         metadataCallback(metadata);
@@ -538,8 +564,11 @@ function parseJPEGMetadata(file, metadataCallback, metadataError) {
 
       // map exif orientation flags for easy transforms
       switch (exif.ORIENTATION) {
-        case undefined:
         case 1:
+        default:
+          // This is the default orientation. If it is properly encoded
+          // we will get 1 here. But sometimes it is undefined and some
+          // files have a 0 here as well.
           metadata.rotation = 0;
           metadata.mirrored = false;
           break;
@@ -571,8 +600,6 @@ function parseJPEGMetadata(file, metadataCallback, metadataError) {
           metadata.rotation = 270;
           metadata.mirrored = false;
           break;
-        default:
-          throw Error('Unknown Exif code for orientation');
       }
     }
   }
@@ -771,6 +798,7 @@ function parseJPEGMetadata(file, metadataCallback, metadataError) {
     return null;
   }
 }
+
 'use strict';
 
 //
@@ -914,6 +942,11 @@ function getVideoRotation(blob, rotationCallback) {
     }
   });
 }
+
+/* exported getImageSize */
+/* global BlobView */
+/* global parseJPEGMetadata */
+
 /*
  * Determine the pixel dimensions of an image without actually
  * decoding the image. Passes an object of metadata to the callback
@@ -933,6 +966,8 @@ function getVideoRotation(blob, rotationCallback) {
  * parseJPEGMetadata() function from shared/js/media/jpeg_metadata_parser.js
  */
 function getImageSize(blob, callback, error) {
+  'use strict';
+
   BlobView.get(blob, 0, Math.min(1024, blob.size), function(data) {
     // Make sure we are at least 8 bytes long before reading the first 8 bytes
     if (data.byteLength <= 8) {
@@ -964,10 +999,47 @@ function getImageSize(blob, callback, error) {
         error(e.toString());
       }
     }
+    else if (magic.substring(0, 2) === 'BM' &&
+             data.getUint32(2, true) === blob.size)
+    {
+      // This is a BMP file
+      try {
+        var width, height;
+
+        if (data.getUint16(14, true) === 12) { // check format version
+          width = data.getUint16(18, true);  // 16-bit little endian width
+          height = data.getUint16(20, true); // 16-bit little endian height
+        }
+        else { // newer versions of the format use 32-bit ints
+          width = data.getUint32(18, true);  // 32-bit little endian width
+          height = data.getUint32(22, true); // 32-bit little endian height
+        }
+
+        callback({
+          type: 'bmp',
+          width: width,
+          height: height
+        });
+      }
+      catch (e) {
+        error(e.toString());
+      }
+    }
     else if (magic.substring(0, 2) === '\xFF\xD8') {
       parseJPEGMetadata(blob,
                         function(metadata) {
-                          metadata.type = 'jpeg';
+                          if (metadata.progressive) {
+                            // If the jpeg parser tells us that this is
+                            // a progressive jpeg, then treat that as a
+                            // distinct image type because pjpegs have
+                            // such different memory requirements than
+                            // regular jpegs.
+                            delete metadata.progressive;
+                            metadata.type = 'pjpeg';
+                          }
+                          else {
+                            metadata.type = 'jpeg';
+                          }
                           callback(metadata);
                         },
                         error);
@@ -977,6 +1049,7 @@ function getImageSize(blob, callback, error) {
     }
   });
 }
+
 'use strict';
 
 //
@@ -1008,6 +1081,9 @@ var metadataParser = (function() {
     height: THUMBNAIL_HEIGHT
   };
 
+  // Don't try to decode image files of unknown type if bigger than this
+  var MAX_UNKNOWN_IMAGE_FILE_SIZE = .5 * 1024 * 1024; // half a megabyte
+
   // An <img> element for loading images
   var offscreenImage = new Image();
 
@@ -1146,10 +1222,10 @@ var metadataParser = (function() {
     getImageSize(file, gotImageSize, gotImageSizeError);
 
     function gotImageSizeError(errmsg) {
-      // The image is not a JPEG, PNG or GIF file. We may still be
+      // The image is not a JPEG, PNG, GIF or BMP file. We may still be
       // able to decode and display it but we don't know the image
       // size, so we won't even try if the file is too big.
-      if (file.size > CONFIG_MAX_UNKNOWN_IMAGE_FILE_SIZE) {
+      if (file.size > MAX_UNKNOWN_IMAGE_FILE_SIZE) {
         metadataError('Ignoring large file ' + file.name);
         return;
       }
@@ -1189,9 +1265,22 @@ var metadataParser = (function() {
       // For jpeg images we can downsample while decoding which means that
       // we can handle images that are quite a bit larger.
       //
+      // getImageSize() distinguishes pjpeg files from jpeg, because
+      // progressive jpeg files are quite different than regular jpegs.
+      // Not only do pjpegs not benefit from using #-moz-samplesize, they
+      // actually require more memory to decode than other images of the same
+      // resolution, so we actually need to reduce the image size limit for
+      // this image type.
+      //
       var imagesizelimit = CONFIG_MAX_IMAGE_PIXEL_SIZE;
-      if (file.type === 'image/jpeg')
+      if (metadata.type === 'jpeg') {
         imagesizelimit *= Downsample.MAX_AREA_REDUCTION;
+      }
+      else if (metadata.type === 'pjpeg') {
+        // As a semi-educated guess, we'll say that we can handle pjpegs
+        // 2/3rds the size of the largest PNG we can handle.
+        imagesizelimit *= 2 / 3;
+      }
 
       //
       // Even if we can downsample an image while decoding it, we still
@@ -1218,23 +1307,6 @@ var metadataParser = (function() {
         return;
       }
 
-      // If the image is lower resolution but with large file size, like
-      // animated GIF, we also need to block this case.
-      if (file.size > CONFIG_MAX_IMAGE_FILE_SIZE) {
-        metadataError('Ignoring acceptable resolution but large file ' +
-                      file.name);
-        return;
-      }
-
-      // If the image is lower resolution but with large file size, like
-      // animated GIF, we should not decode it.
-      if ((file.type === 'image/gif' || file.name.endsWith('.gif')) &&
-        file.size > CONFIG_MAX_GIF_IMAGE_FILE_SIZE) {
-        metadataError('Ignoring acceptable resolution but large gif file ' +
-        file.name);
-        return;
-      }
-
       // If the file included a preview image, see if it is big enough
       if (metadata.preview) {
         // Create a blob that is just the preview image
@@ -1265,8 +1337,10 @@ var metadataParser = (function() {
         // at preview size when needed. So in this case, we just need
         // to create a thumbnail for the image.  On the other hand, if
         // this is not a JPEG image then we have to create both a
-        // thumbnail and an external preview image.
-        if (file.type === 'image/jpeg') {
+        // thumbnail and an external preview image. Note that progressive
+        // jpegs will have a type 'pjpeg' and will be handled below
+        // like png images.
+        if (metadata.type === 'jpeg') {
           cropResizeRotate(file, null, thumbnailSize, null, metadata,
                            function gotThumbnail(error, thumbnailBlob) {
                              if (error) {
