@@ -1,4 +1,5 @@
-/*global GestureDetector, Dialog */
+/*global GestureDetector, Dialog, Navigation, SharedComponents, Utils,
+         Settings */
 
 (function(exports) {
   'use strict';
@@ -15,9 +16,6 @@
   var relation = new WeakMap();
   var rdigit = /^\d/;
 
-  // See recipients list container's transform for singleline
-  const MIN_HEIGHT = 45;
-
   function Recipient(opts) {
     var number;
 
@@ -26,11 +24,13 @@
     this.number = (opts.number || '') + '';
     this.email = opts.email || '';
     this.editable = opts.editable || 'true';
+    this.role = this.editable ? 'textbox' : 'button';
     this.source = opts.source || 'manual';
     this.type = opts.type || '';
-    this.separator = opts.separator || '';
     this.carrier = opts.carrier || '';
     this.className = 'recipient';
+    this.isEmail = Settings.supportEmailRecipient &&
+                   Utils.isEmailAddress(this.number);
 
     // isLookupable
     //  the recipient was accepted by pressing <enter>
@@ -55,7 +55,9 @@
     // is questionable and may be invalid.
     number = this.number[0] === '+' ? this.number.slice(1) : this.number;
 
-    if (this.source === 'manual' && !rdigit.test(number)) {
+    if (this.isEmail) {
+      this.className += ' email';
+    } else if (this.source === 'manual' && !rdigit.test(number)) {
       this.isQuestionable = true;
     }
 
@@ -63,7 +65,7 @@
     // mark it visually for the user.
     //
     if (this.isQuestionable || this.isInvalid) {
-      this.className += ' attention';
+      this.className += ' invalid';
     }
   }
 
@@ -104,8 +106,7 @@
 
   Recipient.FIELDS = [
     'name', 'number', 'email', 'editable', 'source',
-    'type', 'separator', 'carrier',
-    'isQuestionable', 'isInvalid', 'isLookupable'
+    'type', 'carrier', 'isQuestionable', 'isInvalid', 'isLookupable'
   ];
 
   /**
@@ -178,8 +179,8 @@
     });
 
     data.set(this, list);
+    events.set(this, { add: [], remove: [], modechange: [] });
     view.set(this, new Recipients.View(this, setup));
-    events.set(this, { add: [], remove: [] });
   }
 
   /**
@@ -377,6 +378,7 @@
     var template = setup.template;
     var nodes = [];
     var clone;
+    var outerCss = window.getComputedStyle(outer);
 
     priv.set(this, {
       owner: owner,
@@ -389,7 +391,8 @@
         isTransitioning: false,
         visible: 'singleline'
       },
-      minHeight: MIN_HEIGHT
+      minHeight: parseInt(outerCss.getPropertyValue('min-height'), 10),
+      mode: 'singleline-mode'
     });
 
     clone = inner.cloneNode(true);
@@ -416,6 +419,8 @@
       }
     });
 
+    this.updateMode = Utils.debounce(this.updateMode, 100);
+
     ['click', 'keypress', 'keyup', 'blur', 'pan'].forEach(function(type) {
       outer.addEventListener(type, this, false);
     }, this);
@@ -540,12 +545,14 @@
       // in the recipients list.
       node.isPlaceholder = false;
       node.contentEditable = false;
+      node.setAttribute('role', 'button');
 
       // The last node should be contentEditable=true
       // and isPlaceholder=true
       if (i === nodes.length - 1) {
         node.isPlaceholder = true;
         node.contentEditable = true;
+        node.setAttribute('role', 'textbox');
       } else {
         // Map the node to it's entry in the list
         // (only for actual recipient nodes)
@@ -553,10 +560,12 @@
       }
     });
 
-    if (view.state.visible === 'singleline') {
+    if (view.state.visible === 'singleline' && nodes.length) {
       inner.querySelector(':last-child').scrollIntoView(false);
     }
 
+    this.updateMode();
+
     return this;
   };
 
@@ -605,9 +614,37 @@
       // scroll to the bottom of the inner view
       view.inner.scrollTop = view.inner.scrollHeight;
     }
+
+    this.updateMode();
+
     return this;
   };
 
+  /**
+   * Checks whether recipients view mode (single or multi line) has changed
+   * since previous check and notifies listeners with 'modechange' event in this
+   * case.
+   */
+  Recipients.View.prototype.updateMode = function() {
+    var view = priv.get(this);
+
+    var mode = view.inner.scrollHeight > view.minHeight ?
+        'multiline-mode' : 'singleline-mode';
+
+    if (view.mode !== mode) {
+      view.mode = mode;
+
+      // the list in "singleline-mode" should only have "singleline" view
+      if (mode === 'singleline-mode' && view.state.visible === 'multiline') {
+        this.visible('singleline', {
+          refocus: this
+        });
+      }
+
+      view.owner.emit('modechange', mode);
+    }
+  };
+
   var rtype = /^(multi|single)line$/;
 
   Recipients.View.prototype.visible = function(type, opts) {
@@ -654,11 +691,12 @@
 
     // Once the transition has ended, the set focus to
     // the last child element in the recipients list view
-    view.inner.parentNode.addEventListener('transitionend', function te() {
+    view.outer.addEventListener('transitionend', function te() {
       var last = view.inner.lastElementChild;
       var previous;
 
-      if (location.hash === '#new' && state.visible === 'singleline') {
+      if (Navigation.isCurrentPanel('composer') &&
+          state.visible === 'singleline') {
         while (last !== null && last.isPlaceholder) {
           previous = last.previousElementSibling;
           if (!last.textContent) {
@@ -670,9 +708,7 @@
         if (opts.refocus) {
           opts.refocus.focus();
         }
-      }
-
-      if (last !== null) {
+      } else if (state.visible === 'multiline' && last !== null) {
         last.scrollIntoView(true);
       }
 
@@ -892,7 +928,7 @@
           isDeletingRecipient = true;
         }
 
-
+        this.updateMode();
         break;
 
       case 'keypress':
@@ -982,6 +1018,7 @@
           number: typed,
           editable: editable,
           source: 'manual',
+          role: editable ? 'textbox' : 'button',
           isLookupable: isLookupable
         });
 
@@ -999,6 +1036,7 @@
     if (isEdittingRecipient) {
       // Make the last added entry "editable"
       target.contentEditable = true;
+      target.setAttribute('role', 'textbox');
       target.isPlaceholder = true;
       this.focus(target);
     }
@@ -1032,20 +1070,13 @@
       };
 
       // build fragment for dialog body
-      var dialogBody = document.createDocumentFragment();
-      if (recipient.type) {
-        var typeElement = document.createElement('span');
-        if (!navigator.mozL10n.get(recipient.type)) {
-          typeElement.textContent = recipient.type;
-        } else {
-          navigator.mozL10n.localize(typeElement, recipient.type);
-        }
-        dialogBody.appendChild(typeElement);
-      }
+      var dialogBody = document.createElement('div');
 
-      dialogBody.appendChild(document.createTextNode(
-        recipient.separator + recipient.carrier + recipient.number
-      ));
+      dialogBody.innerHTML = SharedComponents.phoneDetails({
+        number: recipient.number,
+        type: recipient.type,
+        carrier: recipient.carrier
+      });
 
       // Dialog will have a closure reference to the response
       // object, therefore it's not necessary to pass it around
@@ -1056,7 +1087,7 @@
             value: recipient.name || recipient.number
           },
           body: {
-            value: dialogBody
+            value: dialogBody.firstElementChild
           },
           options: {
             cancel: {
@@ -1073,7 +1104,7 @@
                 response.isConfirmed = true;
                 handler();
               },
-              position: 'left'
+              className: 'danger'
             }
           }
         });
