@@ -2,25 +2,26 @@
 
 var assert = require('assert');
 
-requireGaia('/test_apps/test-agent/common/test/synthetic_gestures.js');
-var MarionetteHelper = requireGaia('/tests/js-marionette/helper.js');
+requireGaia('/dev_apps/test-agent/common/test/synthetic_gestures.js');
 
 var PerformanceHelper =
   requireGaia('/tests/performance/performance_helper.js');
 var DialerIntegration = require('./integration.js');
 
-marionette(mozTestInfo.appPath + ' >', function() {
+marionette(config.appPath + ' >', function() {
   var client = marionette.client({
     settings: {
-      'ftu.manifestURL': null
+      'ftu.manifestURL': null,
+      'lockscreen.enabled': false
     }
   });
+  // Do nothing on script timeout. Bug 987383
+  client.onScriptTimeout = null;
 
   setup(function() {
-    this.timeout(500000);
-    client.setScriptTimeout(50000);
-
-    MarionetteHelper.unlockScreen(client);
+    this.timeout(config.timeout);
+    client.setScriptTimeout(config.scriptTimeout);
+    PerformanceHelper.injectHelperAtom(client);
   });
 
   test('Dialer/callLog rendering time >', function() {
@@ -34,21 +35,26 @@ marionette(mozTestInfo.appPath + ' >', function() {
       lastEvent: lastEvent
     });
 
+    performanceHelper.unlockScreen();
+
     performanceHelper.repeatWithDelay(function(app, next) {
       var waitForBody = true;
       app.launch(waitForBody);
 
-      performanceHelper.observe();
-
       app.element('optionRecents', function(err, recentsButton) {
         recentsButton.tap();
       });
 
-      performanceHelper.waitForPerfEvent(function(runResults) {
-        performanceHelper.reportRunDurations(runResults);
+      performanceHelper.waitForPerfEvent(function(runResults, error) {
+        if (error) {
+          app.close();
+          throw error;
+        } else {
+          performanceHelper.reportRunDurations(runResults, 'start-call-log');
 
-        assert.ok(Object.keys(runResults).length, 'empty results');
-        app.close();
+          assert.ok(Object.keys(runResults).length, 'empty results');
+          app.close();
+        }
       });
 
     });
